﻿import "@syncfusion/ej2-base/styles/material.css";
import "@syncfusion/ej2-navigations/styles/material.css";
import "@syncfusion/ej2-popups/styles/material.css";
import "@syncfusion/ej2-splitbuttons/styles/material.css";
import "@syncfusion/ej2-react-grids/styles/material.css";
import "@syncfusion/ej2-buttons/styles/material.css";
import "../icons.css";

import * as React from "react";
import { DataManager, ODataV4Adaptor, Query } from "@syncfusion/ej2-data";
import { DialogComponent } from "@syncfusion/ej2-react-popups";
import { GridComponent, Page, Filter, Sort, ColumnsDirective, ColumnDirective, Inject } from "@syncfusion/ej2-react-grids";
import { TabComponent, TabItemsDirective, TabItemDirective } from "@syncfusion/ej2-react-navigations";
import { ButtonComponent, } from "@syncfusion/ej2-react-buttons"
import { TextBoxComponent, FormValidator } from "@syncfusion/ej2-react-inputs";
import { DropDownListComponent } from "@syncfusion/ej2-react-dropdowns";

import { default as UnitNewDialog } from "../unit/UnitNewDialog";
import { default as AssemblySequenceUsageGrid } from "../assembly/AssemblySequenceUsageGrid";

import { getHeader } from "../../components/auth/localStorage"

const axios = require("axios");

export default class SequenceAttachMeasureDialog extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            tolerancemeasure: {
                MeasureMeasureId: 0,
                MeasureUnitUnitId: 0,
                MeasureResourcesResourcesId: 0,
                MeasureResourcesItalianText: "",
                MeasureResourcesEnglishText: "",
                MeasureResourcesFrenchText: "",
                MeasureResourcesGermanText: "",
                ToleranceToleranceId: 0,
                ToleranceLow: "",
                ToleranceNominal: 5,
                ToleranceHigh: "",
            },

            TolerancePerc: 10,
            isVisible: true,
        }

        this.header = getHeader();

        axios.defaults.headers.authorization = this.header.Authorization;
        axios.defaults.headers.crossDomain = true;

        const API_ROOT = window.app && window.app.env.API_URL;

        this.attachUrl = API_ROOT + "/sequences/" + this.props.SequenceId + "/attach";

        this.MeasureAPI = API_ROOT + "/measures";
        this.MeasureData = new DataManager({ url: this.MeasureAPI, adaptor: new ODataV4Adaptor(), headers: [this.header] });
        this.MeasureQuery = new Query().select(["MeasureId", "UnitUnitId", "ResourcesItalianText"]);

        this.UnitAPI = API_ROOT + "/units";
        this.UnitData = new DataManager({ url: this.UnitAPI, adaptor: new ODataV4Adaptor(), headers: [this.header] });
        this.UnitQuery = new Query().select(["UnitId", "ResourcesItalianText"]).sortBy('ResourcesItalianText');

        this.Dialogs = [];

        this.languagesField =
            [
                "MeasureResourcesItalianText",
                "MeasureResourcesFrenchText",
                "MeasureResourcesGermanText",
                "MeasureResourcesEnglishText",
            ];
    }

    componentDidMount() {
        this.toleranceFormValidator = new FormValidator("#toleranceform", {
            rules: {
                'ToleranceLow': { required: true, number: true },
                'ToleranceNominal': { required: true, number: true },
                'ToleranceHigh': { required: true, number: true },
                'TolerancePerc': { number: true, min: 0 },
            },
        });

        this.newMeasureRules = {
            'MeasureMeasureId': { required: false },
            'MeasureUnitUnitId': { required: true },
            'MeasureResourcesItalianText': {required:true, minLength: 3},
            'MeasureResourcesEnglishText': {required:true, minLength: 3},
            'MeasureResourcesGermanText': {required:true, minLength: 3},
            'MeasureResourcesFrenchText': {required:true, minLength: 3},
        };

        this.existingMeasureRules = { 'MeasureMeasureId': { required: true, number: true },};

        this.measureFormValidator = new FormValidator("#measureform", {
            rules: this.newMeasureRules,
        });
    }

    componentDidUpdate() {
        this.updateRules();
        this.toleranceFormValidator.validate();
        this.measureFormValidator.validate();
    }

    updateRules = () => {
        if (this.toleranceFormValidator.validate("ToleranceLow"))
        {
            this.toleranceFormValidator.addRules("ToleranceNominal", { min: this.state.tolerancemeasure.ToleranceLow });
        }
        if (this.toleranceFormValidator.validate("ToleranceNominal"))
        {
            this.toleranceFormValidator.addRules("ToleranceLow", { max: this.state.tolerancemeasure.ToleranceNominal });
            this.toleranceFormValidator.addRules("ToleranceHigh", { min: this.state.tolerancemeasure.ToleranceNominal });
        }
        if (this.toleranceFormValidator.validate("ToleranceHigh"))
        {
            this.toleranceFormValidator.addRules("ToleranceNominal", { max: this.state.tolerancemeasure.ToleranceHigh });
        }
    };

    resetForm = () => {
        const temp = {
            MeasureMeasureId: 0,
            MeasureUnitUnitId: 0,
            MeasureResourcesResourcesId: 0,
            MeasureResourcesItalianText: "",
            MeasureResourcesEnglishText: "",
            MeasureResourcesFrenchText: "",
            MeasureResourcesGermanText: "",
            ToleranceToleranceId: 0,
            ToleranceLow: "",
            ToleranceNominal: 5,
            ToleranceHigh: "",
        };

        this.setState({ tolerancemeasure: temp });
        this.measureGrid.clearSelection();
    }

    submitForm = () => {
        const tol = this.toleranceFormValidator.validate();
        const meas = this.measureFormValidator.validate();

        if (tol && meas) {
            this.submitbutton.disabled = true;
            axios.post(this.attachUrl, this.state.tolerancemeasure)
                .then(response => {
                    let text = "Measure successfully attached to Sequenece " + this.props.SequenceId + "!";
                    alert(text);
                    console.log(text);

                    this.resetForm();
                }, error => {
                    alert("Error: There was missing data in the request or the Measure was already used in this sequence.");
                    console.error(error);
                }).then(() => this.submitbutton.disabled = false);
        }
    }

    openSequenceUsageDialog = (args) => {
        args.preventDefault();

        this.Dialogs.push(<DialogComponent
            width={1000}
            height={650}
            showCloseIcon={true}
            heightAdjustMode="Auto"
            key={Math.random()}
            isModal={true}
            header={"Assemblies using Sequence " + this.props.SequenceId}
            allowDragging={true}
        >
            <AssemblySequenceUsageGrid SequenceId={this.props.SequenceId} />
        </DialogComponent>);
        this.setState({ DialogCount: 1 + this.state.DialogCount });
    }

    close = () => {
        this.props.afterClosing();
        this.setState({ isVisible: false });
    }

    handleComboChange = (args) => {
        if (args.isInteracted) {
            const tm = this.state.tolerancemeasure;

            const key = args.element.id;
            const value = args.value;

            tm[key] = value;
            this.setState({ tolerancemeasure: tm });
        }
    }

    handleChange = (args) => {
        if (args.isInteracted) {
            const tm = this.state.tolerancemeasure;

            const key = args.event.target.id;
            const value = args.value;

            tm[key] = value;
            this.setState({ tolerancemeasure: tm });
        }
    }

    handleOtherDataChange = (args) => {
        if (args.isInteracted) {
            const key = args.event.target.id;
            const value = args.value;
            this.setState({ [key]: value });
        }
    }

    openAddUnitDialog = () => {
        this.Dialogs.push(<UnitNewDialog key={Math.random()} afterSuccess={this.refreshUnits} />);
        this.setState({ DialogCount: 1 + this.state.DialogCount });
    }

    refreshUnits = () => {
        this.units.refresh();
    }

    copyLanguages = (args, name) => {
        console.log(args);

        args.preventDefault();

        if (!this.measureFormValidator.validate(name)) return;

        var tm = this.state.tolerancemeasure;
        const value = tm[name];

        if (value !== null) {
            this.languagesField.forEach((lang) => {
                if (lang !== "" && lang !== name) {
                    tm[lang] = value;
                }
            });
            this.setState({ tolerancemeasure: tm });
        }

    }

    calcTolerances = (args) => {
        args.preventDefault();


        if (!this.toleranceFormValidator.validate("TolerancePerc")) return;

        const perc = parseFloat(this.state.TolerancePerc)/100;
        const tm = this.state.tolerancemeasure;

        tm.ToleranceLow = tm.ToleranceNominal * (1 - perc);
        tm.ToleranceHigh = tm.ToleranceNominal * (1 + perc);

        if (tm.ToleranceLow > tm.ToleranceHigh) {
            const temp = tm.ToleranceLow;

            tm.ToleranceLow = tm.ToleranceHigh;
            tm.ToleranceHigh = temp;
        }

        if (isNaN(tm.ToleranceLow) || isNaN(tm.ToleranceHigh)) {
            alert("Cannot compute tolerances.");
            return;
        }



        tm.ToleranceHigh = Math.round(tm.ToleranceHigh * 100) / 100;
        tm.ToleranceLow = Math.round(tm.ToleranceLow * 100) / 100;

        this.setState({ tolerancemeasure: tm });
    }

    footerTemplate = () => {
        return (
            <div className="form-row">
                <div className="form-group col-xs-1">
                    <ButtonComponent cssClass="e-round" iconCss="e-icons e-list-icon" isPrimary={false} onClick={this.openSequenceUsageDialog} />
                </div>
                <div className="form-group col-xs-11">
                    <ButtonComponent type="submit" cssClass="e-round" ref={b => this.submitbutton = b} iconCss="e-icons e-save-icon" isPrimary={true} onClick={this.submitForm} />
                </div>
            </div>
        );
    }

    onChangeTab = (e) => {
        if (e.isSwiped) e.cancel = true;

        this.measureFormValidator.removeRules();

        const tm = this.state.tolerancemeasure;

        if (e.selectedIndex === 1) {
            tm.MeasureMeasureId = 0;

            this.measureFormValidator.rules = this.newMeasureRules;
        } else {
            tm.MeasureMeasureId = null;
           
            this.measureFormValidator.rules = this.existingMeasureRules;
        }

        this.setState({ tolerancemeasure: tm });
    }

    onSelectRow = (e) => {
        const tm = this.state.tolerancemeasure;
        tm.MeasureMeasureId = e.data.MeasureId
        this.setState({ tolerancemeasure: tm });
        this.measureFormValidator.validate("MeasureMeasureId");
    }

    newTemplate = (data, data1) => {
        return (
            <div style={{ marginTop: 20 }}>
                <div className="form-row">
                    <div className="form-group col-xs-10">
                        <div className="e-float-input e-control-wrapper">
                            <DropDownListComponent value={data.MeasureUnitUnitId} id="MeasureUnitUnitId" name="MeasureUnitUnitId" dataSource={this.UnitData} query={this.UnitQuery} fields={{ value: "UnitId", text: "ResourcesItalianText" }} placeholder="Unit" floatLabelType="Always" change={(e) => this.handleComboChange(e)} ref={(ddl) => this.units = ddl} />
                        </div>
                    </div>
                    <div className="form-group col-xs-2">
                        <div className="e-float-input e-control-wrapper">
                            <ButtonComponent cssClass="e-round" iconCss="e-icons e-plus-icon" onClick={this.openAddUnitDialog} isPrimary={true} />
                        </div>
                    </div>
                </div>
                <div className="form-row">
                    <div className="form-group col-xs-10">
                        <div className="e-float-input e-control-wrapper">
                            <TextBoxComponent id="MeasureResourcesItalianText" name="MeasureResourcesItalianText" value={data.MeasureResourcesItalianText} change={(e) => this.handleChange(e)} placeholder="Italian" floatLabelType="Auto" />
                        </div>
                    </div>
                    <div className="form-group col-xs-2">
                        <div className="e-float-input e-control-wrapper">
                            <ButtonComponent cssClass="e-round" iconCss="e-icons e-copy-icon" isPrimary={false} onClick={(e) => this.copyLanguages(e, "MeasureResourcesItalianText")} />
                        </div>
                    </div>
                </div>
                <div className="form-row">
                    <div className="form-group col-xs-10">
                        <div className="e-float-input e-control-wrapper">
                            <TextBoxComponent id="MeasureResourcesFrenchText" name="MeasureResourcesFrenchText" value={data.MeasureResourcesFrenchText} change={(e) => this.handleChange(e)} placeholder="French" floatLabelType="Auto" />
                        </div>
                    </div>
                    <div className="form-group col-xs-2">
                        <div className="e-float-input e-control-wrapper">
                            <ButtonComponent cssClass="e-round" iconCss="e-icons e-copy-icon" isPrimary={false} onClick={(e) => this.copyLanguages(e, "MeasureResourcesFrenchText")} />
                        </div>
                    </div>
                </div>
                <div className="form-row">
                    <div className="form-group col-xs-10">
                        <div className="e-float-input e-control-wrapper">
                            <TextBoxComponent id="MeasureResourcesGermanText" name="MeasureResourcesGermanText" value={data.MeasureResourcesGermanText} change={(e) => this.handleChange(e)} placeholder="German" floatLabelType="Auto" />
                        </div>
                    </div>
                    <div className="form-group col-xs-2">
                        <div className="e-float-input e-control-wrapper">
                            <ButtonComponent cssClass="e-round" iconCss="e-icons e-copy-icon" isPrimary={false} onClick={(e) => this.copyLanguages(e, "MeasureResourcesGermanText")} />
                        </div>
                    </div>
                </div>
                <div className="form-row">
                    <div className="form-group col-xs-10">
                        <div className="e-float-input e-control-wrapper">
                            <TextBoxComponent id="MeasureResourcesEnglishText" name="MeasureResourcesEnglishText" value={data.MeasureResourcesEnglishText} change={(e) => this.handleChange(e)} placeholder="English" floatLabelType="Auto" />
                        </div>
                    </div>
                    <div className="form-group col-xs-2">
                        <div className="e-float-input e-control-wrapper">
                            <ButtonComponent cssClass="e-round" iconCss="e-icons e-copy-icon" isPrimary={false} onClick={(e) => this.copyLanguages(e, "MeasureResourcesEnglishText")} />
                        </div>
                    </div>
                </div>
            </div>
        );
    }

    existingTemplate = (data) => {
        return (
            <GridComponent
                allowPaging={true}
                pageSettings={{ pageSize: 8, pageSizes: true }}
                allowFiltering={true}
                filterSettings={{ mode: 'Immediate', immediateModeDelay: 300 }}
                allowSorting={true}
                allowSelection={true}
                selectionSettings={{ type: "Single", mode: "Row" }}
                rowSelecting={this.onSelectRow}
                width="100%"
                height={320}
                dataSource={this.MeasureData}
                query={this.MeasureQuery}
                ref={g => this.measureGrid = g}
            >
                <ColumnsDirective>
                    <ColumnDirective field="MeasureId" headerText="Measure ID" isPrimaryKey={true}></ColumnDirective>
                    <ColumnDirective field="ResourcesItalianText" width="300px" headerText="Measure" ></ColumnDirective>
                    <ColumnDirective field="UnitUnitId" allowFiltering={false} headerText="Unit" dataSource={this.UnitData} query={this.UnitQuery} foreignKeyField="UnitId" foreignKeyValue="Symbol" />
                </ColumnsDirective>

                <Inject services={[Page, Sort, Filter]} />
            </GridComponent>
        );
    }

    render() {
        let headerText = [{ text: "New" }, { text: "Existing" }];

        return (
            <div>
                <DialogComponent
                    width={1300}
                    height={850}
                    showCloseIcon={true}
                    heightAdjustMode="Auto"
                    visible={this.state.isVisible}
                    isModal={true}
                    close={this.close}
                    footerTemplate={this.footerTemplate}
                    header={"Associating Measure to Sequence " + this.props.SequenceId}
                    ref={d => this.Dialog = d}
                    allowDragging={true}
                >
                    <div className="form-row">
                        <div className="col-xs-6">
                            <h5>
                                Measure
                            </h5>
                            <form id="measureform">
                                <div className="form-group">
                                    <div className="e-float-input e-control-wrapper">
                                        <TextBoxComponent enabled={false} value={this.state.tolerancemeasure.MeasureMeasureId} placeholder="Measure Id" id="MeasureMeasureId" name="MeasureMeasureId" floatLabelType="Auto" />
                                    </div>
                                </div>
                                <TabComponent
                                    heightAdjustMode="Auto"
                                    id="MeasureTab"
                                    selecting={this.onChangeTab}
                                >
                                    <TabItemsDirective>
                                        <TabItemDirective
                                            header={headerText[0]}
                                            content={{
                                                template: this.newTemplate,
                                                data: { },
                                            }}
                                        />
                                        <TabItemDirective
                                            header={headerText[1]}
                                            content={{
                                                template: this.existingTemplate,
                                                data: {
                                                    MeasureUnitUnitId: this.state.tolerancemeasure.MeasureUnitUnitId,
                                                    MeasureResourcesEnglishText: this.state.tolerancemeasure.MeasureResourcesEnglishText,
                                                    MeasureResourcesFrenchText: this.state.tolerancemeasure.MeasureResourcesFrenchText,
                                                    MeasureResourcesGermanText: this.state.tolerancemeasure.MeasureResourcesGermanText,
                                                    MeasureResourcesItalianText: this.state.tolerancemeasure.MeasureResourcesItalianText,
                                                },
                                            }}
                                        />
                                    </TabItemsDirective> 
                                </TabComponent>
                            </form>
                        </div>

                        <div className="col-xs-6">
                            <h5>Tolerance</h5>
                            <div style={{ marginTop: 40 }}>
                                <form id="toleranceform">
                                    <div className="form-row">
                                        <div className="form-group col-xs-12">
                                            <div className="e-float-input e-control-wrapper">
                                                <TextBoxComponent format="n4" value={this.state.tolerancemeasure.ToleranceLow} id="ToleranceLow" name="ToleranceLow" change={(e) => this.handleChange(e)} placeholder="Low" floatLabelType="Auto" />
                                            </div>
                                        </div>
                                    </div>
                                    <div className="form-row">
                                        <div className="form-group col-xs-7">
                                            <div className="e-float-input e-control-wrapper">
                                                <TextBoxComponent format="n4" value={this.state.tolerancemeasure.ToleranceNominal} id="ToleranceNominal" name="ToleranceNominal" change={(e) => this.handleChange(e)} placeholder="Nominal" floatLabelType="Auto" />
                                            </div>
                                        </div>
                                        <div className="form-group col-xs-3">
                                            <div className="e-float-input e-control-wrapper">
                                                <TextBoxComponent value={this.state.TolerancePerc} id="TolerancePerc" name="TolerancePerc" change={(e) => this.handleOtherDataChange(e)} placeholder="% Tolerance" floatLabelType="Auto" />
                                            </div>
                                        </div>
                                        <div className="form-group col-xs-1">
                                            <div className="e-float-input e-control-wrapper">
                                                <ButtonComponent cssClass="e-round" iconCss="e-icons e-calc-icon" isPrimary={true} onClick={this.calcTolerances} />
                                            </div>
                                        </div>
                                    </div>
                                    <div className="form-row">
                                        <div className="form-group col-xs-12">
                                            <div className="e-float-input e-control-wrapper">
                                                <TextBoxComponent format="n4" value={this.state.tolerancemeasure.ToleranceHigh} id="ToleranceHigh" name="ToleranceHigh" change={(e) => this.handleChange(e)} placeholder="High" floatLabelType="Auto" />
                                            </div>
                                        </div>
                                    </div>
                                </form>
                            </div>
                        </div>
                    </div>
                </DialogComponent>

                <div>
                    {this.Dialogs}
                </div>
            </div>
        );
    }
}

